(* $Id: rtest.ml,v 1.14 2008/03/14 20:07:08 deraugla Exp $
 *
 * Rogloglo Toolkit: test
 *)

open Rt;

value pack1_desc attr args = pack_desc attr args (fun _ _ -> ());
value button1_desc attr txt = button_desc attr (txt, None);
value comm_desc attr = button1_desc [LeftJustifAtt; BorderAtt 0 :: attr];
value popup_desc attr = pack1_desc [BorderAtt 2 :: attr];

value line_buff = String.create 200;
value max x y = if x > y then x else y
and min x y = if x > y then y else x
and message s = do { print_string s; print_newline () }
and s f g x = f x (g x)
and k x y = x
and b f g x = f (g x)
and add x y = x + y
and read_line ic =
  String.sub line_buff 0
    (input_rec 0 where rec input_rec i =
       match input_char ic with
       [ '\n' -> i
       | c -> do { String.set line_buff i c; input_rec (succ i) } ])
;
value iterate f =
  iterate_f where rec iterate_f n x =
    if n <= 0 then x else iterate_f (pred n) (f x)
;
value implode_ascii l =
  let len = List.length l in
  let s0 = String.make len ' ' in
  let _ =
    iterate
      (fun
       [ (i, [hd :: tl]) -> do { String.set s0 i (Char.chr hd); (i + 1, tl) }
       | _ -> failwith "fail" ])
      len (0, l)
  in
  s0
;

value hex_string hstr =
  let len = String.length hstr
  and hex =
    fun
    [ '0'..'9' as c -> Char.code c - Char.code '0'
    | 'a'..'f' as c -> Char.code c - Char.code 'a' + 10
    | 'A'..'F' as c -> Char.code c - Char.code 'A' + 10
    | _ -> failwith "hex_string" ]
  in
  iter "" 0 where rec iter str i =
    if i >= len then str
    else if String.get hstr i == ' ' then iter str (i + 1)
    else if String.get hstr i == '\n' then iter str (i + 1)
    else if String.get hstr i == ',' then iter str (i + 1)
    else if String.get hstr i == '0' && String.get hstr (i + 1) == 'x' then
      iter
        (str ^
         String.make 1
           (Char.chr
              (hex (String.get hstr (i + 2)) * 16 +
               hex (String.get hstr (i + 3)))))
        (i + 4)
    else failwith "hex_string"
;

value curs_width = 42 and curs_height = 61;
value curs_x = curs_width / 2 and curs_y = curs_height / 2;
value curs_src =
  hex_string "
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00,
 0x00, 0x00, 0x00, 0xf0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x8c, 0x02, 0x00,
 0x00, 0x00, 0x80, 0x83, 0x04, 0x00, 0x00, 0x00, 0x70, 0x80, 0x08, 0x00,
 0x00, 0x00, 0x0c, 0x80, 0x10, 0x00, 0x00, 0x80, 0x03, 0x80, 0x20, 0x00,
 0x00, 0x70, 0x00, 0x80, 0x20, 0x00, 0x00, 0x0c, 0x00, 0x80, 0x20, 0x00,
 0x80, 0x03, 0x00, 0x80, 0x20, 0x00, 0x70, 0x00, 0x00, 0x83, 0x20, 0x00,
 0x10, 0x00, 0xe0, 0x82, 0x20, 0x00, 0x10, 0x00, 0x1c, 0x82, 0x20, 0x00,
 0x10, 0x80, 0x03, 0x82, 0x20, 0x00, 0x10, 0x70, 0x00, 0x82, 0x20, 0x00,
 0x10, 0x0c, 0x00, 0x82, 0x20, 0x00, 0x10, 0x0c, 0x00, 0x82, 0x20, 0x00,
 0x10, 0x14, 0x00, 0x83, 0x20, 0x00, 0x10, 0x24, 0xe0, 0x82, 0x20, 0x00,
 0x10, 0x44, 0x1c, 0x82, 0x20, 0x00, 0x10, 0x84, 0x02, 0x82, 0x20, 0x00,
 0x10, 0x04, 0x01, 0x82, 0x20, 0x00, 0x10, 0x04, 0x01, 0x82, 0x20, 0x00,
 0x10, 0x04, 0x01, 0x82, 0x20, 0x00, 0x10, 0x04, 0x01, 0x82, 0x20, 0x00,
 0x10, 0x04, 0x01, 0x82, 0x20, 0x00, 0x10, 0x04, 0x01, 0x82, 0x20, 0x00,
 0x10, 0x04, 0x01, 0x82, 0x20, 0x00, 0x10, 0x04, 0x01, 0x82, 0x20, 0x00,
 0x10, 0x04, 0x01, 0x82, 0x20, 0x00, 0x10, 0x04, 0x01, 0x82, 0x20, 0x00,
 0x10, 0x04, 0x01, 0x82, 0x20, 0x00, 0x10, 0x04, 0x01, 0x82, 0x20, 0x00,
 0x10, 0x04, 0x01, 0x82, 0x20, 0x00, 0x10, 0x04, 0x01, 0x82, 0x20, 0x00,
 0x10, 0x04, 0x01, 0x82, 0x20, 0x00, 0x10, 0x04, 0x01, 0x82, 0x20, 0x00,
 0x10, 0x04, 0x81, 0x85, 0x20, 0x00, 0x10, 0x04, 0x41, 0x88, 0x20, 0x00,
 0x10, 0x04, 0x31, 0x90, 0x20, 0x00, 0x10, 0x04, 0x0d, 0xa0, 0x20, 0x00,
 0x10, 0x04, 0x03, 0xc0, 0x20, 0x00, 0x10, 0x04, 0x01, 0xc0, 0x20, 0x00,
 0x10, 0x04, 0x01, 0x30, 0x20, 0x00, 0x10, 0x04, 0x01, 0x0c, 0x20, 0x00,
 0x10, 0x04, 0x01, 0x03, 0x20, 0x00, 0x10, 0x04, 0xc1, 0x00, 0x20, 0x00,
 0x10, 0x04, 0x31, 0x00, 0x20, 0x00, 0x10, 0x04, 0x0d, 0x00, 0x30, 0x00,
 0x10, 0x04, 0x03, 0x00, 0x0c, 0x00, 0x10, 0x04, 0x00, 0x80, 0x03, 0x00,
 0x10, 0x04, 0x00, 0x70, 0x00, 0x00, 0x10, 0x04, 0x00, 0x0c, 0x00, 0x00,
 0x20, 0x04, 0x80, 0x03, 0x00, 0x00, 0x40, 0x04, 0x60, 0x00, 0x00, 0x00,
 0x80, 0x04, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x85, 0x03, 0x00, 0x00, 0x00,
 0x00, 0x66, 0x00, 0x00, 0x00, 0x00, 0x00, 0x1c, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
 "
;
value curs_mask =
  hex_string "
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x01, 0x00,
 0x00, 0x00, 0x00, 0xf8, 0x03, 0x00, 0x00, 0x00, 0x00, 0xff, 0x07, 0x00,
 0x00, 0x00, 0xc0, 0xff, 0x0f, 0x00, 0x00, 0x00, 0xf8, 0xff, 0x1f, 0x00,
 0x00, 0x00, 0xff, 0xff, 0x3f, 0x00, 0x00, 0xe0, 0xff, 0xff, 0x7f, 0x00,
 0x00, 0xfc, 0xff, 0xff, 0x7f, 0x00, 0x00, 0xff, 0xff, 0xff, 0x7f, 0x00,
 0xe0, 0xff, 0xff, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0x7f, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0x1f, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0x07, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0x03, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0x83, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0xe3, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0xfb, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00, 0xf8, 0xff, 0xff, 0xff, 0x7f, 0x00,
 0xf8, 0xff, 0xff, 0xff, 0x3f, 0x00, 0xf8, 0xff, 0xff, 0xff, 0x0f, 0x00,
 0xf8, 0xff, 0xff, 0xff, 0x01, 0x00, 0xf8, 0xff, 0xff, 0x7f, 0x00, 0x00,
 0xf0, 0xff, 0xff, 0x0f, 0x00, 0x00, 0xe0, 0xff, 0xff, 0x01, 0x00, 0x00,
 0xc0, 0xff, 0x7f, 0x00, 0x00, 0x00, 0x80, 0xff, 0x0f, 0x00, 0x00, 0x00,
 0x00, 0xff, 0x01, 0x00, 0x00, 0x00, 0x00, 0x7e, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x0c, 0x00, 0x00, 0x00, 0x00
"
;

value button_action act _ =
  fun
  [ ButtonEvRelease _ _ _ -> act ()
  | _ -> () ]
;

value popup_action act wid =
  fun
  [ ButtonEvPress xll yll _ -> act wid xll yll
  | ButtonEvEnter xll yll -> act wid xll yll
  | _ -> () ]
;

value scroll_action act wid =
  fun
  [ ScrollEvButtonPress {item = but} val0 -> act wid but val0
  | ScrollEvButtonMotion val0 -> act wid 0 val0
  | _ -> () ]
;

value term_action key_callb button_callb wid =
  fun
  [ TermEvButtonPress row _ col {item = but} -> button_callb wid but row col
  | TermEvButtonRelease row _ col {item = but} ->
      button_callb wid (-but) row col
  | TermEvButtonMotion row _ col -> button_callb wid 0 row col
  | TermEvKeyPress x -> key_callb wid (term_string_of_keysym wid x)
  | TermEvAnswer x -> key_callb wid x
  | _ -> () ]
;

value run_rt name func = do {
  let xd = rt_initialize name in
  let v = try func xd with x -> do { rt_end xd; raise x } in rt_end xd;
  v
};

value my_scroll_set vmin vmax shift act wid but val0 =
  let oval = scroll_val wid - shift in
  let val0 =
    if but = 1 then oval - 1 else if but = 3 then oval + 1 else val0
  in
  let val0 = max vmin (min vmax val0) in
  if val0 <> oval then do { scroll_set wid (val0 + shift); act val0 } else ()
;

value my_text_button xd wid but lin col =
  match but with
  [ 1 -> term_emphasize_from wid lin col
  | (-1) | 3 -> do {
      term_emphasize_to wid lin col;
      let txt = term_get_emphasized wid in rt_set_cut_buffer xd txt
    }
  | (-2) ->
      let txt = rt_get_cut_buffer xd in term_send wid txt
  | _ -> () ]
;

value action wid_named _ = do {
  message "le bouton \"bouton\" provoque l'appel de cette fonction";
  let main_wid = wid_named "test" in print_string " x=";
  print_int (widget_x main_wid);
  print_string " y=";
  print_int (widget_y main_wid);
  print_string " w=";
  print_int (widget_width main_wid);
  print_string " h=";
  print_int (widget_height main_wid);
  print_newline ()
}
and test_popup wid_named txt =
  button_action (fun () -> term_send (wid_named "T") (txt ^ " "))
and autre_action xd _ =
  let ca_marche wid = message "ca marche " in
  let wid =
    rt_create_widget xd "glop" "glop" AutoPosition None
      (button1_desc [] "bouton" (button_action ca_marche))
  in
  rt_map_widget wid
and quit_fun wid_named _ = do {
  let wid = wid_named "test"
  and wid_q = wid_named "question" in
  let w = widget_width wid
  and h = widget_height wid
  and sw = widget_width wid_q
  and sh = widget_height wid_q in
  rt_move_widget wid_q ((w - sw) / 2) ((h - sh) / 2);
  rt_freeze_widget wid;
  rt_unfreeze_widget wid_q;
  rt_map_widget wid_q
}
and keyp_fun wid c = term_send wid (" \008" ^ c ^ "_\008")
and stdin_fun buff len = do {
  print_string "<";
  print_string (String.sub buff 0 len);
  print_string ">";
  flush stdout
}
and trace_scr val0 = do {
  print_string "scroll ";
  print_string "; val=";
  print_int val0;
  print_newline ()
}
and answer xa wid_named b0 _ =
  if b0 then rt_stop_main_loop xa
  else do {
    rt_unfreeze_widget (wid_named "test");
    rt_unmap_widget (wid_named "question")
  }
;

value sub_popup (txt, wid, lev) =
  comm_desc [] txt
    (popup_action
       (fun bwid llx lly ->
          let x = llx + widget_width bwid
          and y = lly - widget_height bwid in
          rt_map_popup_widget wid x y lev))
and empty_wid xd = rt_create_popup_widget xd (pack1_desc [] (Horizontal, []));

value translate_wid xd =
  let wid_named = widget_named xd in
  let test_popup = test_popup wid_named in
  rt_create_popup_widget xd
    (popup_desc []
       (Vertical,
        [title_desc [] "Hello" (fun _ _ -> ());
         comm_desc [] "this" (test_popup "ceci");
         comm_desc [] "is" (test_popup "est");
         comm_desc [] "a" (test_popup "un");
         comm_desc [] "popup menu" (test_popup "menu deroulant");
         comm_desc [] "" (fun _ _ -> ());
         comm_desc [] "gag"
           (button_action (fun _ -> rt_raise_widget (wid_named "gag")))]))
and gag_wid xd =
  let wid_named = widget_named xd in
  let test_popup = test_popup wid_named in
  rt_create_popup_widget xd
    (popup_desc []
       (Vertical,
        [pack1_desc []
           (Horizontal,
            [title_desc [] "Salut" (fun _ _ -> ());
             comm_desc [] "Ca" (test_popup "my");
             comm_desc [] "marche" (test_popup "tailor");
             comm_desc [] "aussi" (test_popup "is");
             comm_desc [] "horizontalement" (test_popup "rich")]);
         comm_desc [] "ok"
           (button_action
              (fun _ -> rt_raise_widget (wid_named "translate")))]))
and button_wid xd =
  let wid_named = widget_named xd in
  rt_create_popup_widget xd
    (button1_desc [BorderAtt 2] "bouton" (button_action (action wid_named)))
and machines_wid xd service_wid sun_wid workstations_wid =
  let _test_popup = test_popup (widget_named xd) in
  rt_create_popup_widget xd
    (popup_desc []
       (Vertical,
        [sub_popup ("service", service_wid, 1); sub_popup ("sun", sun_wid, 1);
         sub_popup ("workstations", workstations_wid, 1)]))
and service_wid xd =
  let test_popup = test_popup (widget_named xd) in
  rt_create_popup_widget xd
    (popup_desc []
       (Vertical,
        [comm_desc [] "Margaux" (test_popup "Margaux");
         comm_desc [] "Pommard" (test_popup "Pommard");
         comm_desc [] "Seti" (test_popup "Seti");
         comm_desc [] "Nuri" (test_popup "Nuri");
         comm_desc [] "Romanee" (test_popup "Romanee");
         comm_desc [] "Corton" (test_popup "Corton");
         comm_desc [] "Ens" (test_popup "Ens")]))
and sun_wid xd =
  let test_popup = test_popup (widget_named xd) in
  rt_create_popup_widget xd
    (popup_desc []
       (Vertical,
        [comm_desc [] "Beaune" (test_popup "Beaune");
         comm_desc [] "Brouilly" (test_popup "Brouilly");
         comm_desc [] "Cornas" (test_popup "Cornas");
         comm_desc [] "Yquem" (test_popup "Yquem")]))
and workstations_wid xd algo_wid croap_wid formel_wid para_wid ens_wid =
  rt_create_popup_widget xd
    (popup_desc []
       (Vertical,
        [sub_popup ("algo", algo_wid, 2); sub_popup ("croap", croap_wid, 2);
         sub_popup ("formel", formel_wid, 2); sub_popup ("para", para_wid, 2);
         sub_popup ("ens", ens_wid, 2)]))
and algo_wid xd =
  let test_popup = test_popup (widget_named xd) in
  rt_create_popup_widget xd
    (popup_desc []
       (Vertical,
        [comm_desc [] "Amour" (test_popup "Amour");
         comm_desc [] "Bandol" (test_popup "Bandol");
         comm_desc [] "Charmes" (test_popup "Charmes");
         comm_desc [] "Fleurie" (test_popup "Fleurie");
         comm_desc [] "Nuits" (test_popup "Nuits");
         comm_desc [] "Quarts" (test_popup "Quarts");
         comm_desc [] "Tokay" (test_popup "Tokay")]))
and croap_wid xd =
  let test_popup = test_popup (widget_named xd) in
  rt_create_popup_widget xd
    (popup_desc []
       (Vertical,
        [comm_desc [] "Barr" (test_popup "Barr");
         comm_desc [] "Barsac" (test_popup "Barsac");
         comm_desc [] "Fixin" (test_popup "Fixin");
         comm_desc [] "Huaine" (test_popup "Huaine");
         comm_desc [] "Marix" (test_popup "Marix");
         comm_desc [] "Moulis" (test_popup "Moulis");
         comm_desc [] "Muscat" (test_popup "Muscat");
         comm_desc [] "Pogo" (test_popup "Pogo");
         comm_desc [] "Sancerre" (test_popup "Sancerre");
         comm_desc [] "Sylvaner" (test_popup "Sylvaner");
         comm_desc [] "Tobago" (test_popup "Tobago");
         comm_desc [] "Wallis" (test_popup "Wallis");
         comm_desc [] "Zeus" (test_popup "Zeus")]))
and formel_wid xd =
  let wid_named = widget_named xd in
  let test_popup = test_popup wid_named in
  rt_create_popup_widget xd
    (popup_desc []
       (Vertical,
        [comm_desc [] "Ausone" (test_popup "Ausone");
         comm_desc [] "Banyuls" (test_popup "Banyuls");
         comm_desc [] "Bergerac" (test_popup "Bergerac");
         comm_desc [] "Bouzy" (test_popup "Bouzy");
         comm_desc [] "Jurancon" (test_popup "Jurancon");
         comm_desc [] "Latour" (test_popup "Latour");
         comm_desc [] "Madiran" (test_popup "Madiran");
         comm_desc [] "Morgon" (test_popup "Morgon");
         comm_desc [] "Musigny" (test_popup "Musigny");
         comm_desc
           [BackgroundAtt
              (ColorPn
                 (if is_colored xd then rt_create_color xd (255, 0, 0)
                  else rt_white_color xd))]
           "Peray" (test_popup "Peray");
         comm_desc [] "" (button_action (fun () -> ()));
         comm_desc [] "Pernand" (test_popup "Pernand");
         comm_desc [] "Petrus" (test_popup "Petrus");
         comm_desc [] "Pomerol" (test_popup "Pomerol");
         comm_desc [] "Pouilly" (test_popup "Pouilly");
         comm_desc [] "Quincy" (test_popup "Quincy");
         comm_desc [] "Reuilly" (test_popup "Reuilly");
         comm_desc [] "Riesling" (test_popup "Riesling");
         comm_desc [] "Rieussec" (test_popup "Rieussec");
         comm_desc [] "Santenay" (test_popup "Santenay");
         comm_desc [] "Vougeot" (test_popup "Vougeot")]))
and para_wid xd =
  let test_popup = test_popup (widget_named xd) in
  rt_create_popup_widget xd
    (popup_desc []
       (Vertical,
        [comm_desc [] "Bellet" (test_popup "Bellet");
         comm_desc [] "Chablis" (test_popup "Chablis");
         comm_desc [] "Couchey" (test_popup "Couchey");
         comm_desc [] "Givry" (test_popup "Givry");
         comm_desc [] "Kpriss" (test_popup "Kpriss");
         comm_desc [] "Volnay" (test_popup "Volnay");
         comm_desc [] "Saumur" (test_popup "Saumur")]))
and ens_wid xd =
  let test_popup = test_popup (widget_named xd) in
  rt_create_popup_widget xd
    (popup_desc []
       (Vertical,
        [comm_desc [] "Aneth" (test_popup "Aneth");
         comm_desc [] "Ares" (test_popup "Ares");
         comm_desc [] "Asterix" (test_popup "Asterix");
         comm_desc [] "Basilic" (test_popup "Basilic");
         comm_desc [] "Berlioz" (test_popup "Berlioz");
         comm_desc [] "Bouleau" (test_popup "Bouleau");
         comm_desc [] "Cannelle" (test_popup "Cannelle");
         comm_desc [] "Cassis" (test_popup "Cassis");
         comm_desc [] "Cedre" (test_popup "Cedre");
         comm_desc [] "Crocus" (test_popup "Crocus");
         comm_desc [] "Debussy" (test_popup "Debussy");
         comm_desc [] "Diamant" (test_popup "Diamant");
         comm_desc [] "Ens" (test_popup "Ens");
         comm_desc [] "Fuchsia" (test_popup "Fuchsia");
         comm_desc [] "Gentiane" (test_popup "Gentiane");
         comm_desc [] "Hermes" (test_popup "Hermes");
         comm_desc [] "Jasmin" (test_popup "Jasmin");
         comm_desc [] "Magnolia" (test_popup "Magnolia");
         comm_desc [] "Menthe" (test_popup "Menthe");
         comm_desc [] "Merisier" (test_popup "Merisier");
         comm_desc [] "Mimosa" (test_popup "Mimosa");
         comm_desc [] "Muscade" (test_popup "Muscade");
         comm_desc [] "Myrtille" (test_popup "Myrtille");
         comm_desc [] "Obelix" (test_popup "Obelix");
         comm_desc [] "Oseille" (test_popup "Oseille");
         comm_desc [] "Ravel" (test_popup "Ravel");
         comm_desc [] "Rubis" (test_popup "Rubis");
         comm_desc [] "Safran" (test_popup "Safran");
         comm_desc [] "Saphir" (test_popup "Saphir");
         comm_desc [] "Sibelius" (test_popup "Sibelius");
         comm_desc [] "Verdi" (test_popup "Verdi")]))
;

value rec make_test_widget xa xd translate_wid gag_wid machines_wid
    button_wid = do {
  let wid_named = widget_named xd in
  let nVIS = 10
  and nSAV = 30 in
  let font_fixed = rt_load_query_font xd "fixed" in
  let wid =
    rt_create_widget xd "test" "test" AutoPosition None
      (pack1_desc [NameAtt "test"]
         (Horizontal,
          [pack1_desc [BorderAtt 0; BandAtt 0]
             (InDepth,
              [pack1_desc [NameAtt "S1"]
                 (Vertical,
                  [comm_desc [FillerAtt] "bouton"
                     (button_action (action wid_named));
                   button1_desc [] "pustule"
                     (button_action (autre_action xd));
                   pack1_desc [FillerAtt]
                     (Vertical,
                      [button1_desc [] "trans & pos"
                         (button_action
                            (fun _ -> rt_raise_widget (wid_named "S4")));
                       button1_desc [] "button attributes"
                         (button_action
                            (fun _ -> rt_raise_widget (wid_named "S5")));
                       button1_desc [] "restart"
                         (button_action
                            (fun _ -> do {
                               rt_destroy_widget (widget_named xd "test");
                               rt_destroy_widget (widget_named xd "trans");
                               let wid =
                                 make_test_widget xa xd translate_wid gag_wid
                                   machines_wid button_wid
                               in
                               rt_map_widget wid
                             }));
                       button1_desc [] "surprise"
                         (button_action
                            (fun _ -> rt_raise_widget (wid_named "S2")))])]);
               pack1_desc [NameAtt "S2"]
                 (Vertical,
                  [button1_desc [] "next"
                     (button_action
                        (fun _ -> rt_raise_widget (wid_named "S3")));
                   pack1_desc [FillerAtt] (Vertical, []);
                   button1_desc [] "resource"
                     (button_action
                        (fun _ -> do {
                           print_string "program : ";
                           flush stdout;
                           let program = read_line stdin in
                           print_string "option : ";
                           flush stdout;
                           let option = read_line stdin in
                           let val0 = rt_get_default xd program option in
                           print_string "value = ";
                           print_string val0;
                           print_newline ()
                         }));
                   button1_desc [] "return"
                     (button_action
                        (fun _ -> rt_raise_widget (wid_named "S1")))]);
               pack1_desc [NameAtt "S3"]
                 (Vertical,
                  [title_desc [] "Scrolling text" (fun _ _ -> ());
                   button1_desc [] "home"
                     (button_action
                        (fun _ -> term_send (wid_named "st") "\027[H"));
                   button1_desc [] "up"
                     (button_action
                        (fun _ -> do {
                           let wid = wid_named "st" in
                           let val0 =
                             nSAV - scroll_val (wid_named "scr") - 1
                           in
                           let val0 =
                             max 0 (min (term_history_size wid) val0)
                           in
                           term_shift wid val0;
                           scroll_set (wid_named "scr") (nSAV - val0)
                         }));
                   pack1_desc [FillerAtt]
                     (Horizontal,
                      [scroll_desc [NameAtt "scr"] (Vertical, 0, nSAV, nVIS)
                         (scroll_action
                            (my_scroll_set 0 (nSAV - nVIS) nVIS
                               (fun val0 ->
                                  let wid = wid_named "st" in
                                  term_shift wid (nSAV - nVIS - val0))));
                       term_desc [NameAtt "st"] (nVIS, 25, nSAV - nVIS)
                         (term_action term_send (my_text_button xd))]);
                   button1_desc [] "down"
                     (button_action
                        (fun _ -> do {
                           let wid = wid_named "st" in
                           let val0 =
                             nSAV - scroll_val (wid_named "scr") + 1
                           in
                           let val0 =
                             max 0 (min (term_history_size wid) val0)
                           in
                           term_shift wid val0;
                           scroll_set (wid_named "scr") (nSAV - val0)
                         }));
                   button1_desc [] "return"
                     (button_action
                        (fun _ -> rt_raise_widget (wid_named "S2")))]);
               pack1_desc [BorderAtt 0; NameAtt "S4"]
                 (Vertical,
                  [title_desc [] "trans & pos" (fun _ _ -> ());
                   button1_desc [] "timeout"
                     (button_action
                        (fun _ -> do {
                           rt_cancel_timeout xa "";
                           rt_set_timeout xa "" (rt_current_time xa + 1000)
                             (fun _ -> do {
                                let t = ftime ()
                                and wid = wid_named "T" in
                                term_send wid "\n";
                                term_send wid (string_of_int t.time);
                                term_send wid ".";
                                let t = "00" ^ string_of_int t.millitm in
                                let t =
                                  String.sub t (String.length t - 3) 3
                                in
                                term_send wid t
                              })
                         }));
                   button1_desc [] "timer"
                     (button_action
                        (fun _ ->
                           let rec timeout_fun _ = do {
                             let t = ftime ()
                             and wid = wid_named "T" in
                             term_send wid "\n";
                             term_send wid (string_of_int t.time);
                             term_send wid ".";
                             let t = "00" ^ string_of_int t.millitm in
                             let t = String.sub t (String.length t - 3) 3 in
                             term_send wid t;
                             rt_set_timeout xa "" (rt_current_time xa + 1000)
                               timeout_fun
                           }
                           in
                           rt_set_timeout xa "" (rt_current_time xa + 1000)
                             timeout_fun));
                   button1_desc [] "stop timers"
                     (button_action (fun _ -> rt_cancel_timeout xa ""));
                   pack1_desc []
                     (Horizontal,
                      [button1_desc [] "trans"
                         (fun bwid ->
                            fun
                            [ ButtonEvRelease x y _ ->
                                rt_map_transient_widget (wid_named "trans") x
                                  y
                            | _ -> () ]);
                       button1_desc [] "pos"
                         (button_action
                            (fun _ -> rt_map_widget (wid_named "pos")));
                       button1_desc [] "auto"
                         (button_action
                            (fun _ -> rt_map_widget (wid_named "auto")))]);
                   pack1_desc [FillerAtt; BorderAtt 0] (Vertical, []);
                   button1_desc [] "ok"
                     (button_action
                        (fun _ -> rt_raise_widget (wid_named "S1")))]);
               pack1_desc [BorderAtt 0; NameAtt "S5"]
                 (Vertical,
                  [title_desc [] "button attributes" (fun _ _ -> ());
                   button1_desc [HeightAtt 40] "height 40 pixels"
                     (fun _ _ -> ());
                   pack1_desc []
                     (Horizontal,
                      [button1_desc [WidthAtt 125] "a" (fun _ _ -> ());
                       button1_desc [WidthAtt 125] "same size than a"
                         (fun _ _ -> ())]);
                   button1_desc [BorderAtt 6] "border = 6 pixels"
                     (fun _ _ -> ());
                   button1_desc [BorderAtt 0] "border = 0" (fun _ _ -> ());
                   button1_desc [BandAtt 6] "band = 6" (fun _ _ -> ());
                   button1_desc [BandAtt 0] "band = 0" (fun _ _ -> ());
                   button1_desc [BoldAtt 6] "bold = 6" (fun _ _ -> ());
                   button1_desc [BoldAtt 0] "bold = 0" (fun _ _ -> ());
                   button1_desc [FontAtt [| font_fixed |]] "font fixed"
                     (fun _ _ -> ());
                   pack1_desc [FillerAtt; BorderAtt 0] (Vertical, []);
                   button1_desc [] "ok"
                     (button_action
                        (fun _ -> rt_raise_widget (wid_named "S1")))])]);
           scroll_desc [] (Vertical, 100, 150, 0)
             (scroll_action (my_scroll_set 100 150 50 trace_scr));
           scroll_desc [] (Vertical, 817, 819, 0)
             (scroll_action (my_scroll_set 817 819 0 trace_scr));
           scroll_desc [] (Vertical, 0, 100, 0)
             (scroll_action (fun wid but val0 -> scroll_set wid val0));
           scroll_desc [] (Vertical, 0, 100, 0)
             (scroll_action
                (fun wid but val0 -> scroll_set wid (val0 + 100)));
           pack1_desc []
             (Vertical,
              [pack1_desc []
                 (Horizontal,
                  [pack1_desc [BorderAtt 0; BandAtt 0]
                     (InDepth,
                      [button1_desc [NameAtt "translate"] "translate"
                         (popup_action
                            (fun bwid xll yll ->
                               rt_map_popup_widget translate_wid xll yll 0));
                       button1_desc [NameAtt "gag"] "gag"
                         (popup_action
                            (fun bwid xll yll ->
                               rt_map_popup_widget gag_wid xll yll 0))]);
                   button1_desc [] "machines"
                     (popup_action
                        (fun bwid xll yll ->
                           rt_map_popup_widget machines_wid xll yll 0));
                   button1_desc [] "bouton"
                     (popup_action
                        (fun bwid xll yll ->
                           rt_map_popup_widget button_wid xll yll 0))]);
               term_desc [NameAtt "T"; FillerAtt] (1, 15, 0)
                 (term_action keyp_fun
                    (fun wid but row col -> do {
                       term_send wid "\027[";
                       term_send wid (string_of_int (row + 1));
                       term_send wid ";";
                       term_send wid (string_of_int (col + 1));
                       term_send wid "f"
                     }))]);
           pack1_desc []
             (Vertical,
              [button1_desc [] "salut"
                 (button_action
                    (fun _ ->
                       let wid = wid_named "test" in
                       rt_resize_widget wid (widget_width wid + 10)
                         (widget_height wid + 10)));
               scroll_desc [] (Horizontal, -3, 10, 3)
                 (scroll_action (my_scroll_set 0 10 0 trace_scr));
               pack1_desc [FillerAtt] (Vertical, []);
               button1_desc [] "hello"
                 (button_action
                    (fun _ -> term_send (wid_named "T") "ca marche\n"))]);
           button1_desc [] "quit" (button_action (quit_fun wid_named))]))
  in
  term_send (wid_named "st") "\027[20h";
  term_send (wid_named "T") "\027[?35h\027[20h";
  scroll_set (wid_named "scr") nSAV;
  rt_select_mouse (wid_named "T")
    (rt_create_bitmap_mouse xd curs_src curs_mask curs_width curs_height
       (0, 0, 255) (0, 255, 0) curs_x curs_y);
  let _ =
    rt_create_transient_widget wid "trans" (Some rt_unmap_widget)
      (pack1_desc [NameAtt "trans"]
         (Horizontal,
          [term_desc [FillerAtt] (2, 80, 0)
             (term_action term_send (my_text_button xd));
           button1_desc [] "OK"
             (button_action (fun _ -> rt_unmap_widget (wid_named "trans")));
           button1_desc [] "Abort"
             (button_action (fun _ -> rt_unmap_widget (wid_named "trans")))]))
  in
  let _ =
    rt_create_subwidget wid 0 0
      (pack1_desc [NameAtt "question"]
         (Vertical,
          [title_desc [] " Are you sure ? " (fun _ _ -> ());
           pack1_desc []
             (Horizontal,
              [button1_desc [] "yes"
                 (button_action (answer xa wid_named True));
               button1_desc [] "cancel"
                 (button_action (answer xa wid_named False))])]))
  in
  wid
};

value test dname =
  run_rt dname
    (fun xd -> do {
       let xa = rt_args [xd] in
       let translate_wid = translate_wid xd
       and gag_wid = gag_wid xd
       and button_wid = button_wid xd
       and machines_wid =
         machines_wid xd (service_wid xd) (sun_wid xd)
           (workstations_wid xd (algo_wid xd) (croap_wid xd) (formel_wid xd)
              (para_wid xd) (ens_wid xd))
       in
       rt_select_file xa 0 stdin_fun;
       let _ : widget =
         rt_create_widget xd "pos" "pos" (UserPosition 40 20) None
           (button1_desc [NameAtt "pos"] "pos"
              (button_action
                 (fun () -> rt_unmap_widget (widget_named xd "pos"))))
       and _ : widget =
         rt_create_widget xd "auto" "auto" AutoPosition None
           (button1_desc [NameAtt "auto"] "auto"
              (button_action
                 (fun () -> rt_unmap_widget (widget_named xd "auto"))))
       in
       let wid =
         make_test_widget xa xd translate_wid gag_wid machines_wid button_wid
       in
       rt_map_widget wid;
       rt_main_loop xa
     })
;
